#!/usr/bin/env node
var prompt = require('prompt');
var program = require('commander');
var package = require('./package.json');
var fs = require('fs');
var AdmZip = require('adm-zip');
var bsdiff = require('node-bsdiff');
var currentPath = process.cwd();
var path = require('path');
var output = "output";
var temp = "temp";

require('shelljs/global');

if (!which('git')) {
    echo('Sorry, this script requires git');
    exit(1);
}

program
    .version(package.version)
    .option('-b, --bundle', 'generate bundle')
    .option('-d, --diffZip [old] [new] [dest]', 'generate diff zip')
    .parse(process.argv);

program
    .on('--help', function(){
});

if (program.bundle) {
    prompt.start();
    prompt.get([{
        name: 'platform',
        description: 'platform (Android or IOS)',
        type: 'string',
        required: true
    }], function(err, result) {
        generateBundle(result.platform.toLowerCase());
    });
};

if (program.diffZip) {
    var args = program.args;
    if (args.length != 2) {
        console.log('arguments error');
    }
    var origin = program.diffZip;
    var next = args[0];
    var output = args[1];
    if (isZipFile) {
        diffZip(origin, next, output);
    }
}

function isZipFile(file) {
    return /\.zip/.test(oldFile);
}

function generateBundle(platform) {
    var cdate = new Date() - 0;
    rm('-rf', currentPath + '/' + output + '/' + temp);
    mkdir('-p', currentPath + '/' + output + '/' + temp);
    exec('react-native bundle ' +
        '--entry-file ' +
        'index.' + platform +'.js ' +
        '--bundle-output ' +
        './' + output + '/' + temp + '/index.bundlejs ' +
        '--platform ' + platform + ' ' +
        '--assets-dest  ' + output + '/' + temp + ' ' +
        '--dev false');
    cd(currentPath + '/' + output + '/' + temp);
    exec('zip -r ' + platform + '.bundle.' + cdate + '.zip *');
    mv('-f', platform + '.bundle.' + cdate + '.zip', '../');
    cd(currentPath);
    echo('build success');
}

function diffZip(origin, next, output) {
    var originZip = new AdmZip(origin);
    var originZipEntries = originZip.getEntries();
    var originData = {};
    var originMap = {};
    mkdir('-p', path.join(currentPath, output));
    originZipEntries.forEach(function(zipEntry) {
        originData[zipEntry.entryName] = zipEntry.getData();
        originMap[zipEntry.entryName] = zipEntry.entryName;
    });

    var nextZip = new AdmZip(next);
    var nextZipEntries = nextZip.getEntries();
    nextZipEntries.forEach(function(zipEntry) {
        if (zipEntry.entryName === 'index.bundlejs') {
            var diff = bsdiff.diff(originData[zipEntry.entryName], zipEntry.getData());
            addEntry(path.join(output, 'index.bundlejs.patch'), diff);
        }

        if (!originData[zipEntry.entryName]) {
            addEntryDirectory(path.join(output, zipEntry.entryName));
        }
    });
}

function addEntry(output, data) {
    touch(output);
    fs.writeFile(output, data, function(err) {
        console.log(err);
    });
}

function addEntryDirectory(output) {
    mkdir('-p', output);
}

